@@ -74,7 +74,8 @@ from schooltool import table
 
 from schooltool.common import SchoolToolMessage as _
 from schooltool.basicperson.browser.person import FlourishPersonIDCardsViewBase
-from schooltool.report.browser.report import RequestReportDownloadDialog
+from schooltool.report.report import OldReportTask
+from schooltool.report.browser.report import RequestRemoteReportDialog
 
 
 class GroupContainerAbsoluteURLAdapter(BrowserView):
@@ -271,15 +272,10 @@ class FlourishGroupTableFormatter(table.table.SchoolToolTableFormatter):
         directlyProvides(title, ISortableColumn)
         return [title]
 
-    def render(self):
-        formatter = self._table_formatter(
-            self.context, self.request, self._items,
-            columns=self._columns,
-            batch_start=self.batch.start, batch_size=self.batch.size,
-            sort_on=self._sort_on,
-            prefix=self.prefix)
+    def makeFormatter(self):
+        formatter = table.table.SchoolToolTableFormatter.makeFormatter(self)
         formatter.cssClasses['table'] = 'groups-table relationships-table'
-        return formatter()
+        return formatter
 
 
 class FlourishGroupListView(EditRelationships):
@@ -810,10 +806,10 @@ class FlourishManageGroupsOverview(flourish.page.Content):
         return IGroupContainer(self.schoolyear, None)
 
 
-class FlourishRequestGroupIDCardsView(RequestReportDownloadDialog):
+class FlourishRequestGroupIDCardsView(RequestRemoteReportDialog):
 
-    def nextURL(self):
-        return absoluteURL(self.context, self.request) + '/group_id_cards.pdf'
+    report_builder = 'group_id_cards.pdf'
+    report_task = OldReportTask
 
 
 class FlourishGroupIDCardsView(FlourishPersonIDCardsViewBase):
@@ -823,6 +819,12 @@ class FlourishGroupIDCardsView(FlourishPersonIDCardsViewBase):
         return _('ID Cards for Group: ${group}',
                  mapping={'group': self.context.title})
 
+    @property
+    def filename(self):
+        sy = ISchoolYear(self.context.__parent__)
+        return 'id_cards_%s_%s.pdf' % (
+            self.context.title,  sy.title)
+
     def persons(self):
         result = [self.getPersonData(person)
                   for person in self.context.members]
@@ -857,3 +859,37 @@ class GroupAwarePersonTableFilter(PersonTableFilter):
 
     template = ViewPageTemplateFile('templates/f_group_aware_person_table_filter.pt')
 
+
+class PersonGroupsTable(table.table.TableContent):
+
+    group_by_column = 'schoolyear'
+
+    @Lazy
+    def source(self):
+        int_ids = getUtility(IIntIds)
+        source = dict([
+                ('%s-%d' % (group.__name__, int_ids.getId(group)), group)
+                for group in self.context.groups
+                ])
+        return source
+
+    def columns(self):
+        default = table.table.TableContent.columns(self)
+        description = zc.table.column.GetterColumn(
+            name='description',
+            title=_('Description'),
+            getter=lambda i, f: i.description or '',
+            )
+        schoolyear = SchoolYearColumn(
+            name='schoolyear',
+            title=_(u'School Year'),
+            subsort=True)
+        directlyProvides(schoolyear, ISortableColumn)
+        return default + [description, schoolyear]
+
+
+class PersonProfileGroupsPart(table.pdf.RMLTablePart):
+
+    table_name = "groups_table"
+    title = _("Group memberships")
+
