@@ -22,7 +22,6 @@ SchoolTool report pages.
 import cgi
 import datetime
 import re
-import urllib
 
 try:
     import Image
@@ -42,6 +41,7 @@ from z3c.rml import rml2pdf
 from schooltool.app import pdf
 from schooltool.app.interfaces import ISchoolToolApplication
 from schooltool.app.interfaces import IApplicationPreferences
+from schooltool.skin.flourish.helpers import quoteFilename
 from schooltool.skin.flourish import content
 from schooltool.skin.flourish import interfaces
 from schooltool.skin.flourish import form
@@ -74,16 +74,6 @@ class Box(object):
             self.bottom = bottom
 
 
-def quoteFilename(filename):
-    if not filename:
-        return filename
-    if type(filename) is unicode:
-        encoded = filename.encode('UTF-8')
-    else:
-        encoded = str(filename)
-    return urllib.quote(encoded)
-
-
 class PDFPage(page.PageBase):
     implements(interfaces.IPDFPage)
 
@@ -103,6 +93,11 @@ class PDFPage(page.PageBase):
     pdf_disabled_text = _("PDF support is disabled."
                           "  It can be enabled by your administrator.")
 
+    # TODO: Should return True when running tests
+    render_invariant = False
+    # TODO: Should return True when devmode enabled
+    render_debug = False
+
     def renderPDF(self, xml):
         filename = self.filename
         stream = rml2pdf.parseString(xml, filename=filename or None)
@@ -137,16 +132,6 @@ class PDFPage(page.PageBase):
     def filename(self):
         return self.makeFileName(self.base_filename)
 
-    @property
-    def render_debug(self):
-        # TODO: Should return True when devmode enabled
-        return False
-
-    @property
-    def render_invariant(self):
-        # TODO: Should return True when running tests
-        return False
-
     def __call__(self):
         if not pdf.isEnabled():
             return translate(self.pdf_disabled_text, context=self.request)
@@ -163,6 +148,9 @@ class PDFPage(page.PageBase):
 
 class IPlainPDFPage(interfaces.IPDFPage):
 
+    message_title = zope.schema.TextLine(
+        title=u"Short report description", required=False)
+
     name = zope.schema.TextLine(
         title=u"Report name", required=False)
 
@@ -187,6 +175,7 @@ class PlainPDFPage(PDFPage):
     title = None
     subtitles_left = None
     subtitles_right = None
+    message_title = _('report')
 
 
 class PDFInitSection(viewlet.ViewletManager):
