@@ -179,9 +179,6 @@ function fillPopupMenu(link) {
     var data = link.data('popup-menu-data');
     var popup = link.prev();
     popup.empty();
-    if (!data) {
-        return;
-    }
     var header = $('<li class="header"></li>');
     header.text(data.header);
     popup.append(header);
